// Fichier main.c
/////////////////////////////////////////////////////////

/****************************************************************
 *                                                    			*
 * .Date de cration : 11/01/08                         		*
 * .Auteur : Guibert David                              		*       
 *           Cadic Matthieu										*
 *           Luzeau Yohan                                		*
 * .Descriptions :                                      		*
 *      - Fichier principal pour la gestion du rouleau droit   	*
 *                                                      		*
 *                                                      		*
 *                                                      		*
 * .Remarques :                                         		*
 *      - Gestion des ordre reus par interruption plutt		*
 *		  que par polling								   		*
 *      - Adresse de la carte droite 0x4E          				*
 * 		- Transmission IC sur interruption (priorit haute)	*
 * 		- Timer sur interruption (priorit basse)               *
 *																*
 ****************************************************************/

// La led AMBRE est allume pendant l'interruption srie (rception d'un caractre)
// La led JAUNE est allume lorsqu'une commande est en cours 
// La led ROUGE est allume si un blocage est dtect (la vitesse linaire relle ne colle pas au modle)






// ------------ Directives de compilation
#pragma config WDT = OFF	// Pas de  watchdog
#pragma config OSC = HS		// Quartz HS pour un quartz 20MHz
#pragma config LVP = OFF	// Low voltage programming disabled

// --------------------------------------------------------------------------
// Enttes ncessaires

#include <p18f452.h>		// Pour les registres du PIC18F252
#include <math.h>			// Pour les calculs de trigo
#include <delays.h>    		// Pour les temporisations
#include <timers.h>			// Pour la base de temps de 10ms
#include <adc.h>			// Bibliothque des convertisseurs Analogiques Numriques
#include <stdio.h>			// Pour les printf sur la liaison srie




#include "led.h"			// Librairie poue les led de la carte
#include "serial.h"			// Librairie pour la liaison srie
#include "codeur.h"			// Librairie pour les codeurs
#include "moteur.h"			// Librairie pour les moteurs
#include "structure.h"		// Libraisie pour la struture de donnes
#include "trigo.h"			// Librairie pour les calcul de trigo (spcialement l'atan2)
#include "PI.h"


// --------------------------------------------------------------------------
// Constantes

// Mensuration du robot
#define 	r 				28.45					// Rayon de la roue codeuse en mm
#define 	l 				133.25//134.4			// Demi entraxe entre les deux roues codeuse en mm

// Constante pour le modele gometrique
#define		PI				3.141592653589793		// Constante Pi
#define 	ANGLE_IMP800 	0.007853981634 			// 2pi/nb imp par tour

#define 	Kpos 			35.0					// Coefficient de l'asservissement en position
#define		Kangle			90.0					// Coefficient proportionnel de l'asservissement angulaire
#define		KIa				12.0					// Coefficient intgrale de l'asservissement angulaire
#define 	ANGLE_INTEGRALE	0.3						// Erreur angulaire en dessous de laquelle on charge l'intgrale

#define 	K1				((PI*r)/(2.0*400.0))	// Coef pour les coordonnes X, Y 
#define 	K2				((PI*r)/(2.0*400.0*l))	// Coef pour le differentiel de l'angle du robot

#define 	ANGLE_PRECISION	0.01					// Prcision dsire pour l'asservissement angulaire
#define 	ANGLE_ASSERVISS	0.4						// Prcision dsire pour l'asservissement angulaire

#define		ACC_MAX_AV			1.7					// Acclration max en marche avant
#define		DEC_MAX_AV			2.0					// Dcclration max en marche avant
#define		ACC_MAX_AR			1.3					// Acclration max en marche arrire
#define		DEC_MAX_AR			1.0					// Acclration max en marche avant

#define		DEC_URGENCE_AV		5.0					// Dcclration d'urgence en marche avant
#define		DEC_URGENCE_AR		3.0					// Dcclration d'urgence en marche arrire


// --------------------------------------------------------------------------
// Variables partage

extern unsigned char 	CopieTrame[MAX_TRAME];		// Copie de la trame reue par l'I2C
extern char 			Serial_Flag;				// Flag qui indique que la trame est complte

// --------------------------------------------------------------------------


// Prototype de la routine de service d'interuptions de priorit haute
void Interrupt_High_Priority(void);					// Utilise pour l'IC


// Prototype de la routine de service d'interuptions de priorit basse
void Interrupt_Low_Priority(void);					// Utilise pour le timer




/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//                            Fonction Principale                          //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////

void main (void)
{
	// Variables du modle gomtrique
	
	struct 			position Pos_Robot;				// Position/orientation courante du robot 
	int 			DirectGauche=0;					// Commande  appliquer sur le moteur gauche en direct
	int 			DirectDroite=0;					// Commande  appliquer sur le moteur droit en direct	
	unsigned int	Codeur_val_G;					// Valeur brute du codeur gauche
 	unsigned int 	Codeur_val_D;					// Valeur brute du codeur droit 	
	unsigned int 	Codeur_val_G_vieux=0;			// Ancienne valeur du codeur gauche
	unsigned int 	Codeur_val_D_vieux=0;			// Ancienne caleur du codeur droit
	int 			Vitesse_Mesuree_D=0;			// Vitesse du moteur droit que l'on mesure par odomtrie
	int 			Vitesse_Mesuree_G=0;			// Vitesse du moteur gauche que l'on mesure par odomtrie
	
	// Variables de l'asservissement
	
	char 			State='A';						// Etat courant de la carte propulsion
	struct 			point 	Pos_Cible;				// Structure qui contient les coordonnes de la cible
	float			dx,dy;							// Distance en x et en y entre le robot et la cible
	float			Teta=0;							// Angle entre le robot et la cible	
	int 			Vit_Moy=0;						// Vitesse moyenne du dplacement vers la cible
	float 			Consigne_Angle;					// Angle dsir pendant l'asservissement en orientation
	float 			Consigne_Vitesse_Angle;			// Consigne en vitesse angulaire pour l'asservissement en orientation
	float			dAngle;							// Erreur angulaire pour l'asservissement en angle
	float			Vit_Max;						// Vitesse maximum de rotation pendant l'asservissement en position angulaire
	int				Distance;						// Distance  parcourir pour la commande ToutDroit
	float			Distance_Cible;					// Distance entre le robot et la cible	
	float 			Vit_Courante=0;					// Consigne de vitesse linaire courante du robot
	float			Vit_Dec=0;						// Vitesse de dclration
	float			Vit_Angulaire_Courante;			// Vitesse angulaire courante du robot
	
	// Variables pour la liaison srie
			
	unsigned char 	TrameEnvoi[8]; 					// Trame contenant les caractre  envoyer sur la liaison srie
	int				Angle_Temporaire;				// Entier contenant la valeur temporaire de l'angle reu
	
	

	// Configure le micro et initialise les priphriques
	
	Led_Init();								// Initialise les leds
	Serial_Init();							// Initialise la liaison srie
 	Codeur_Init();							// Initialise les codeurs
 	Moteur_Init();							// Initialise les moteurs

	RCONbits.IPEN=1;						// Autorise les niveau de priorit sur les interuptions
	
	// Configuration des interruptions	
	
	INTCONbits.PEIE=0;						// Interdit les interruptions de basse priorit
	INTCONbits.GIE=1;						// Autorise les interruptions de haute priorit


	// Initialise les coordonnes du robot 
	
	Pos_Robot.coordonnee.X=0;											// Se place en 0 en X
	Pos_Robot.coordonnee.Y=0;											// Se place en 0 en Y
	Pos_Robot.orientation=0;											// Se place en 0 angulaire
	Codeur_Lire_val( &Codeur_val_G_vieux,&Codeur_val_D_vieux);			// Initialise les anciennes valeurs des codeurs
	
	PI_Init();
		
	while(1)															// Boucle principale
	{
		
		
// T=0		
		// Lecture des codeurs et mise  jour du modle gomtrique
		
		Codeur_Lire_val( &Codeur_val_G,&Codeur_val_D);					// Lit les codeurs et sauvegarde dans les variables	
		
		Vitesse_Mesuree_G= Codeur_val_G - Codeur_val_G_vieux;			// On rcupere le nombre d'impulsions depuis la derniereIT du moteur Gauche
		Vitesse_Mesuree_D= Codeur_val_D_vieux - Codeur_val_D; 			// On rcupere le nombre d'impulsions depuis la derniereIT du moteur Droit
		
		Codeur_val_G_vieux=Codeur_val_G;								// Sauvegarde de la valeur du codeur Gauche actuel pour la prochaine IT 
		Codeur_val_D_vieux=Codeur_val_D;								// Sauvegarde de la valeur du codeur Droit actuel pour la prochaine IT 
	
		Pos_Robot.coordonnee.X+=K1*(Vitesse_Mesuree_D+Vitesse_Mesuree_G)*cos(Pos_Robot.orientation);	// Position du robot en X
		Pos_Robot.coordonnee.Y+=K1*(Vitesse_Mesuree_D+Vitesse_Mesuree_G)*sin(Pos_Robot.orientation);	// Position du robot en Y
		Pos_Robot.orientation +=K2*(Vitesse_Mesuree_D-Vitesse_Mesuree_G); 								// Calcul du differentiel de l'angle du robot
	 		
		if(Pos_Robot.orientation>PI)	Pos_Robot.orientation-=2.0*PI;	// Garde PSI entre -PI et PI
		if(Pos_Robot.orientation<=-PI)	Pos_Robot.orientation+=2.0*PI;
			
			
			
		
		// Traitement de l'tat courant pour la propulsion
		
		switch (State) {
			
			case 'A':												// Commande arrt
				Moteur_CommandeG(0);								// Envoie la commande sur le moteur gauche
				Moteur_CommandeD(0);								// Envoie la commande sur le moteur droit		
				Vit_Courante=0;
				Led_Jaune(0);										// Eteinds la led jaune : on est au repos
				Led_Rouge(0);										// Eteinds la led rouge : plus de blocage
				State='a';											// On passe dans l'tat d pour indiquer que la commande est en cours
				break;
			
		
			case 'D':												// Commande directe
				Moteur_CommandeG(DirectGauche);						// Envoie la commande sur le moteur gauche
				Moteur_CommandeD(DirectDroite);						// Envoie la commande sur le moteur droit
				State='d';											// On passe dans l'tat d pour indiquer que la commande est en cours
				break;		
		
		
			case 'p':
			case 'P':												// On envoie les commandes polaires 
				if (PI_Vitesse((int)Vit_Courante,(int)(Vit_Angulaire_Courante))>NSampleError)	
					State='p';										// Si a force, on change d'tat
				break;
				
					
			case 'b':
			case 'B':												// On envoie les commandes polaires 
				PI_Direct_Vitesse(DirectGauche,DirectDroite);				
				break;
			
			
			
			case 'R': 	
				if (Vit_Courante<10 && Vit_Courante>-10) 			// On roule suffisamment lentement
				{
					State='A';										// On passe dans l'tat Arret
					break;											// On quitte l'tat 'R'
				}
				
				if (Vit_Courante>0)									// Si la vitesse est positive
					Vit_Courante-=DEC_URGENCE_AV; //10				// On dclre en marche avant
				else												// sinon ..
					Vit_Courante+=DEC_URGENCE_AR; //8				// On dclre en marche arrire
				PI_Vitesse((int)Vit_Courante,0);					// On envoie les consignes sur l'asservissement
				break;												
		
		
			case 'g': Led_Rouge(1);
			case 'G':												// Commande dplacement au point X,Y
				
				dx=Pos_Cible.X-Pos_Robot.coordonnee.X;				// Calcul l'cart en x entre le robot et le point
				dy=Pos_Cible.Y-Pos_Robot.coordonnee.Y;				// Calcul l'cart en y entre le robot et le point
				Distance_Cible=sqrt(dx*dx + dy*dy);					// Distance entre le robot et la cible
				
				if(dx!=0 || dy!=0) 									// On vrifie qu'on est pas pile poil sur le point (sinon erreur atan2)
				{												
										
					Teta=Trigo_atan2(dy,dx)-Pos_Robot.orientation;	// Calcul de l'angle entre le robot et le point traqu
					
					while (Teta>PI)  Teta-=2.0*PI;					// On garde l'angle entre PI et -PI
					while (Teta<-PI) Teta+=2.0*PI;	
					
					if(Vit_Moy<0)									//si on a une vitesse negative On traque en marche arriere
					{				 						
																	// si le point est devant le robot
																	// Si on est arriv au point traqu en marche arriere						
						if (Teta<=(PI/2.0) && Teta>=(-PI/2.0) && Distance_Cible<100.0) 
						{	
							Moteur_CommandeG(0);					// Arret du moteur Gauche
							Moteur_CommandeD(0);					// Arret du moteur Droitt	
							State='A';
						}
						else 
						{
								
							if (Teta<=(2.0*PI/3.0) && Teta>=(-2.0*PI/3.0))	// Si la cible est devant, on tourne sur place
							{
								Vit_Courante+=1;							// Ralentie le robot
								if (Vit_Courante>-20) Vit_Courante=-20;		// Vrifie qu'on ne passe pas dans les positifs !

							}
							else
							{
								Vit_Dec= - Distance_Cible * 0.45;	//0.5	// Calcul la vitesse pour la dclration
								if (Vit_Dec>-20) Vit_Dec=-20;				// Pour garantir un minimum de dplacement
	
								if (fabs (Vit_Courante-Vit_Moy)<2)			// Si la vitesse courante est proche de la vitesse moyenne
									Vit_Courante=Vit_Moy;					// On fusionne
								else if (Vit_Courante>Vit_Moy)			
									Vit_Courante-=ACC_MAX_AR;				// Rampe d'acclration
								else Vit_Courante+=DEC_MAX_AR;				// Rampe de dcclration
								
								if (Vit_Courante<Vit_Dec)					// Si la vitesse courante est supprieur  la vitesse de dclration
									Vit_Courante=Vit_Dec;					// C'est la dclration qui l'emporte

/*								Vit_Courante-=1.8;							// Augmente la vitesse courante (Rampe d'acclration)	
															
								if (Vit_Courante<Vit_Dec)					// Si la vitesse courante est supprieure  la vitesse de dclration
									Vit_Courante=Vit_Dec;					// C'est la dclration qui l'emporte
								else if (Vit_Moy>Vit_Courante)				// Regarde si on est en vitesse de croisire ou en vitesse d'approche
								{
	//								Vit_Courante=Vit_Moy;					// On est en vitesse de croisire
									Vit_Courante+=2;						// On est en vitesse de croisire
									if (Vit_Courante<Vit_Moy) 
											Vit_Courante=Vit_Moy;
								}
								*/
							}	
							
							Teta=PI-Teta;									// Ajoute PI
							while (Teta>PI)  Teta-=2.0*PI;					// On garde l'angle entre PI et -PI
							while (Teta<=-PI) Teta+=2.0*PI;	
							

							if (Distance_Cible>30.0)						// Si on n'est pas trop prs de la cible, 
								Vit_Angulaire_Courante=-Kpos*Teta;			// On calcul de la vitesse angulaire
							else											// sinon
								Vit_Angulaire_Courante=0;					// On va tout droit
								
							if (Vit_Angulaire_Courante> 70) 				// Bride la vitesse angulaire
								Vit_Angulaire_Courante= 70;
							if (Vit_Angulaire_Courante<-70) 
								Vit_Angulaire_Courante=-70;															
							
							if (PI_Vitesse((int)Vit_Courante,(int)(Vit_Angulaire_Courante))>NSampleError)	
								State='g';								// Si a force, on change d'tat
								
						}
					}
					else
					{
						// si le point est derriere le robot
																	// Si on est arriv au point traqu 			
						if ((Teta>(PI/2.0) || Teta<(-PI/2.0)) && Distance_Cible<100.0)				
						{
								Moteur_CommandeG(0);					// Arret du moteur Gauche
								Moteur_CommandeD(0);					// Arret du moteur Droit
								State='A';								// Actualise l'tat -> Arret
						}
						else											// Si on est pas arriv au point traqu -> asservissement
						{
							if (Teta>(PI/3.0) || Teta<(-PI/3.0))		// Si la cible est derrire le robot
							{
								Vit_Courante-=1;						// Ralentie le robot
								if (Vit_Courante<20) Vit_Courante=20;		// Vrifie qu'on n'a pas une vitesse trop faible
							}
							else
							{
								Vit_Dec=Distance_Cible * 0.45; //0.5		// Calcul la vitesse pour la dclration
								if (Vit_Dec<10) Vit_Dec=10;				// Pour garantir un minimum de dplacement
								
								if (fabs (Vit_Courante-Vit_Moy)<2)		// Si la vitesse courante est proche de la vitesse moyenne
									Vit_Courante=Vit_Moy;				// On fusionne
								else if (Vit_Courante>Vit_Moy)			
									Vit_Courante-=DEC_MAX_AV;			// Rampe de dcclration
								else Vit_Courante+=DEC_MAX_AV;			// Rampe d'acclration
								
								if (Vit_Courante>Vit_Dec)				// Si la vitesse courante est supprieur  la vitesse de dclration
									Vit_Courante=Vit_Dec;				// C'est la dclration qui l'emporte

								
/*								Vit_Courante+=1.8;						// Augmente la vitesse courante (Rampe d'acclration)
								
								if (Vit_Courante>Vit_Dec)				// Si la vitesse courante est supprieur  la vitesse de dclration
									Vit_Courante=Vit_Dec;				// C'est la dclration qui l'emporte
								else if (Vit_Moy<Vit_Courante)			// Regarde si on est en vitesse de croisire ou en vitesse d'approche
								{
//									Vit_Courante=Vit_Moy;				// On est en vitesse de croisire
									Vit_Courante-=2;					// On est en vitesse de croisire
									if (Vit_Courante>Vit_Moy) 
										Vit_Courante=Vit_Moy;
								}
*/
							}
							
							if (Distance_Cible>30.0)					// Si on n'est pas trop prs de la cible, 
								Vit_Angulaire_Courante=Kpos*Teta;		// On calcul de la vitesse angulaire
							else										// sinon
								Vit_Angulaire_Courante=0;				// On va tout droit
							
							if (Vit_Angulaire_Courante> 70) 			// Bride la vitesse angulaire
								Vit_Angulaire_Courante= 70;	
							if (Vit_Angulaire_Courante<-70)
								 Vit_Angulaire_Courante=-70;				
											
																		// Envoie les consignes sur le PI
							if (PI_Vitesse((int)Vit_Courante,(int)(Vit_Angulaire_Courante))>NSampleError)	
								State='g';								// Si a force, on change d'tat
						}
					}
				}
				else									  				// Si x=0 alors atan2 tend vers l'infini
				{
					Moteur_CommandeG(0);								// Arret du moteur Gauche
					Moteur_CommandeD(0);								// Arret du moteur Droit
					State='A';											// Actualise l'tat de robot
				}				
				break;	




				
			case 'o': Led_Rouge(1);	
			case 'O':												// Asservissement en position
							
				dAngle=Consigne_Angle-Pos_Robot.orientation;		// Calcul l'erreur angulaire
				while (dAngle>PI)  dAngle-=2.0*PI;					// On garde l'angle entre PI et -PI
				while (dAngle<-PI) dAngle+=2.0*PI;		
				
																
				if (fabs(dAngle)>ANGLE_PRECISION)					// Tant qu'on est pas  la prcision dsire, on asservit
				{
					Consigne_Vitesse_Angle=Kangle*dAngle;
					if (Consigne_Vitesse_Angle>	Vit_Max) Consigne_Vitesse_Angle=Vit_Max;
					if (Consigne_Vitesse_Angle<-Vit_Max) Consigne_Vitesse_Angle=-Vit_Max;
					
					if (PI_Vitesse(0,(int)Consigne_Vitesse_Angle)>NSampleError)
						State='o';
					
				}
				else
				{
					Moteur_CommandeG(0);							// Arret du moteur Gauche
					Moteur_CommandeD(0);							// Arret du moteur Droit
					State='A';										// Actualise l'tat de robot
				}	
				break;	
			
		}
		
		
		
		// Traitement des donnes reues de la liaison srie		
		
		if (Serial_Flag)											// Une trame a t reue
		{
									
		
			switch (CopieTrame[0]) {									// Selon le premier caractre de la trame
			
			
			
			
				case 'O' :												// Commande Oriente (Oriente le robot vers l'angle Consigne_Angle)
					if (State!='O' && State!='o') PI_Stand_By();		// On rinitialise le PI
					
					if (CopieTrame[7]==CRC (CopieTrame,7))				// Vrifie le CRC de la trame
					{
																		// Calcule l'orientation dsire
						Angle_Temporaire=CopieTrame[1]*256 + CopieTrame[2];
						Consigne_Angle=(float)Angle_Temporaire/1000;	// Divise par 1000 et converti en flottant
																		// Calcule la vitesse maximum de rotation
						Vit_Max =(float)(CopieTrame[3]*256 + CopieTrame[4]);
												
						Led_Jaune(1);									// Allume la led, une commande est en cours
						PI_Init();										// Initialise l'asservissement
						State='O';										// Passe dans l'tat asservissement en orientation
						
												
						TrameEnvoi[0]='O';								// Prparation de la trame d'envoi 'O' pour orientation
						TrameEnvoi[1]='O';								// Puis OK
						TrameEnvoi[2]='K';
						Envoyer_Trame(TrameEnvoi,3);					// Envoi la trame pour acquitter la commande
					}
					Serial_Flag=0;										// Fin du traitement de la copie de la trame
					break;
						
						
						
						
						
						
				case 'G' :												// Commande Goto (va au point X,Y  la vitesse Vit_Moy)
				
					if (CopieTrame[7]==CRC (CopieTrame,7))				// Vrifie le CRC de la trame
					{
						Pos_Cible.X=CopieTrame[1]*256 + CopieTrame[2];	// Calcule la position en X de la cible
						Pos_Cible.Y=CopieTrame[3]*256 + CopieTrame[4];	// Calcule la position en Y de la cible
						Vit_Moy	   =CopieTrame[5]*256 + CopieTrame[6];	// Calcule la vitesse moyenne de dplacement
						
						if (Vit_Moy>0 && Vit_Courante<0.0)				// Si le signe de la vitesse moyenne est diffrent du signe de la vitesse courante
							Vit_Courante=0;								// On rinitialise
						if (Vit_Moy<0 && Vit_Courante>0.0)				
							Vit_Courante=0;
							
						Led_Jaune(1);									// Allume la led, une commande est en cours
						if (State!='G' && State!='g') 					// Si on est pas dj dans l'tat GotoXY
							PI_Reset();									// On rinitialise le PI
							
						State='G';										// Passe dans l'tat asservissement en position
												
						TrameEnvoi[0]='G';								// Prparation de la trame d'envoi 'G' pour force
						TrameEnvoi[1]='O';								// Puis OK
						TrameEnvoi[2]='K';
						Envoyer_Trame(TrameEnvoi,3);					// Envoi la trame pour acquitter la commande
					}
					Serial_Flag=0;										// Fin du traitement de la copie de la trame
					break;
					
			
			
			
		
				case 'T' :												// Commande Tout droit
				
					if (CopieTrame[7]==CRC (CopieTrame,7))				// Vrifie le CRC de la trame
					{
						Distance=CopieTrame[1]*256 + CopieTrame[2];		// Calcul la distance  parcourir
						Vit_Moy	=CopieTrame[3]*256 + CopieTrame[4];		// Calcule la vitesse du dplacement			
												
						Pos_Cible.X =Pos_Robot.coordonnee.X;			// Calcule la position en X de la cible
						if (Vit_Moy>0)
							Pos_Cible.X+=Distance*cos(Pos_Robot.orientation);
						else
							Pos_Cible.X-=Distance*cos(Pos_Robot.orientation);
						
						Pos_Cible.Y =Pos_Robot.coordonnee.Y;			// Calcule la position en Y de la cible
						if (Vit_Moy>0)
							Pos_Cible.Y+=Distance*sin(Pos_Robot.orientation);
						else
							Pos_Cible.Y-=Distance*sin(Pos_Robot.orientation);						
							

						
						
						
						if (Vit_Moy>0 && Vit_Courante<0.0)				// Si le signe de la vitesse moyenne est diffrent du signe de la vitesse courante
							Vit_Courante=0;								// On rinitialise
						if (Vit_Moy<0 && Vit_Courante>0.0)				
							Vit_Courante=0;
						
						if (State!='G' && State!='g') 					// Si on est pas dj dans l'tat GotoXY
							PI_Reset();									// On rinitialise le PI
						State='G';										// Passe dans l'tat asservissement en position : suivi de cible
						Led_Jaune(1);									// Allume la led, une commande est en cours
												
						TrameEnvoi[0]='T';								// Prparation de la trame d'envoi 'G' pour force
						TrameEnvoi[1]='O';								// Puis OK
						TrameEnvoi[2]='K';
						Envoyer_Trame(TrameEnvoi,3);					// Envoi la trame pour acquitter la commande


					}
					Serial_Flag=0;										// Fin du traitement de la copie de la trame
					break;
		
			
			
			
				case 'F' :												// Commande Force (Force la position du robot)
				
					if (CopieTrame[7]==CRC (CopieTrame,7))				// Vrifie le CRC de la trame
					{
						Pos_Robot.coordonnee.X=CopieTrame[1]*256 + CopieTrame[2];	// Calcule la nouvelle position en X
						Pos_Robot.coordonnee.Y=CopieTrame[3]*256 + CopieTrame[4];	// Calcule la nouvelle position en Y
						Pos_Robot.orientation =CopieTrame[5]*256 + CopieTrame[6];	// Calcule la nouvelle orientation
						Pos_Robot.orientation/=1000;								// Divise par mille (entier-flottant)						
												
						TrameEnvoi[0]='F';								// Prparation de la trame d'envoi 'F' pour force
						TrameEnvoi[1]='O';								// Puis OK
						TrameEnvoi[2]='K';
						Envoyer_Trame(TrameEnvoi,3);					// Envoi la trame pour acquitter la commande
					}
					Serial_Flag=0;										// Fin du traitement de la copie de la trame
					break;
				
				
				
				
				
				
				case 'S' : 												// Demande de lecture des donnes de la carte (Etat + position + orientation)
						
					if (CopieTrame[7]==CRC(CopieTrame,7))				// Vrifie le CRC de la trame
					{	
										
						TrameEnvoi[0]=State;							// Envoie l'tat courant du processeur
						TrameEnvoi[1]=((int)Pos_Robot.coordonnee.X&0xFF00)>>8;			// Envoie l'octet de poids fort de la cooordonne en X
						TrameEnvoi[2]=(int)Pos_Robot.coordonnee.X&0x00FF;				// Envoie l'octet de poid faible
						TrameEnvoi[3]=((int)Pos_Robot.coordonnee.Y&0xFF00)>>8;			// Envoie l'octet de poids fort de la cooordonne en Y
						TrameEnvoi[4]=(int)Pos_Robot.coordonnee.Y&0x00FF;				// Envoie l'octet de poid faible
						TrameEnvoi[5]=((int)(Pos_Robot.orientation*1000)&0xFF00)>>8;	// Envoie l'octet de poids fort de l'orientation
						TrameEnvoi[6]=(int)(Pos_Robot.orientation*1000)&0x00FF;			// Envoie l'octet de poid faible
						TrameEnvoi[7]=CRC(TrameEnvoi,7);				// Ajout du CRC				
						Envoyer_Trame(TrameEnvoi,8);					// Envoi la trame pour acquitter la commande
					
					}
					Serial_Flag=0;										// Fin du traitement de la copie de la trame
					break;	
				
				
				
				
				
				
				case 'D' :												// Commande Direct
				
					if (CopieTrame[7]==CRC (CopieTrame,7))				// Vrifie le CRC de la trame
					{
						DirectGauche=CopieTrame[1]*256 + CopieTrame[2];	// Calcule la consigne gauche
						DirectDroite=CopieTrame[3]*256 + CopieTrame[4];	// Calcule la consigne droite
						State='D';										// On passe en commande directe
						Led_Jaune(1);									// Allume la led, une commande est en cours
						
						TrameEnvoi[0]='D';								// Prparation de la trame d'envoi 'D' pour direct
						TrameEnvoi[1]='O';								// Puis OK
						TrameEnvoi[2]='K';
						Envoyer_Trame(TrameEnvoi,3);					// Envoi la trame pour acquitter la commande
					}
					Serial_Flag=0;										// Fin du traitement de la copie de la trame
					break;	
					
					
					
					
				case 'P' :												// Commande directe polaire
				
					if (CopieTrame[7]==CRC (CopieTrame,7))				// Vrifie le CRC de la trame
					{
						Vit_Courante=CopieTrame[1]*256 + CopieTrame[2];				// Calcule la consigne en vitesse linaire
						Vit_Angulaire_Courante=CopieTrame[3]*256 + CopieTrame[4];	// Calcule la consigne en vitesse angulaire
						Led_Jaune(1);									// Allume la led, une commande est en cours
						if (State!='P' && State!='p') 					// Si on est pas dj dans l'tat commande polaire
							PI_Reset();									// On rinitialise le PI
						State='P';										// On passe en commande directe

						TrameEnvoi[0]='P';								// Prparation de la trame d'envoi 'D' pour direct
						TrameEnvoi[1]='O';								// Puis OK
						TrameEnvoi[2]='K';
						Envoyer_Trame(TrameEnvoi,3);					// Envoi la trame pour acquitter la commande
					}
					Serial_Flag=0;										// Fin du traitement de la copie de la trame
					break;	

					
					
					
					
					
				case 'A' : 												// Demande d'arret des moteurs (Etat + position + orientation)
										
					if (CopieTrame[7]==CRC(CopieTrame,7))				// Vrifie le CRC de la trame
					{		
						if (State=='G' || State=='g')					// Si on est en train de se dplacer
							State='R';									// On passe dans l'tat de dclration
						else
							State='A';									// Passe dans l'tat Arrt						
						TrameEnvoi[0]='A';								// Prparation de la trame d'envoi 'A' pour ARRET
						TrameEnvoi[1]='O';								// Puis OK
						TrameEnvoi[2]='K';
						Envoyer_Trame(TrameEnvoi,3);					// Envoi la trame pour acquitter la commande				
					}
					Serial_Flag=0;										// Fin du traitement de la copie de la trame
					break;
					
					
				case 'B' :												// Commande Direct
				
					if (CopieTrame[7]==CRC (CopieTrame,7))				// Vrifie le CRC de la trame
					{
						DirectGauche=CopieTrame[1]*256 + CopieTrame[2];	// Calcule la consigne gauche
						DirectDroite=CopieTrame[3]*256 + CopieTrame[4];	// Calcule la consigne droite

						Led_Jaune(1);									// Allume la led, une commande est en cours
						if (State!='B' && State!='b') 					// Si on est pas dj dans l'tat commande directe asservie
							PI_Direct_Reset();							// On rinitialise le PI
						State='B';										// On passe en commande directe
						TrameEnvoi[0]='B';								// Prparation de la trame d'envoi 'D' pour direct
						TrameEnvoi[1]='O';								// Puis OK
						TrameEnvoi[2]='K';
						Envoyer_Trame(TrameEnvoi,3);					// Envoi la trame pour acquitter la commande
					}
					Serial_Flag=0;										// Fin du traitement de la copie de la trame
					break;	
					
					
				default : Serial_Flag=0;								// Ce n'est pas un caractre attendu, fin du traitement
				
			} // Fin switch dialogue avec l'interruption I2C
	
		} // Fin Serial_Flag
				
	} // Fin while 1
}






/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//                            Interruptions                                // 
//                                                                         // 
/////////////////////////////////////////////////////////////////////////////



//----------------------------------------------------------------------------
// Vecteur d'interruptions de priorit haute (utilis pour l'IC)
#pragma code highVector=0x08						// Vecteur d'interruption haut
void atInterruptHigh(void)
{
	_asm goto Interrupt_High_Priority _endasm		// On saute  la fonction MyHighInterrupt
}
#pragma code 										// Retour  la zone de code



//----------------------------------------------------------------------------
// Routine de service de l'interruption haute priorite
#pragma interrupt Interrupt_High_Priority
void Interrupt_High_Priority(void)
{
	
	if (PIR1bits.RCIF) 									// On vrifie qu'in s'agit bien d'une interruption pour la liaison srie
	{	
// T=0
		Led_Ambre(1);									// Allume la led Ambre pour marquer le dbut de l'interruption
		InterruptSerial();								// On appelle la routine d'interruption	
		Led_Ambre(0);									// Eteinds la led pour marquer la fin de l'interruptino
// T=150s
	}	
}





	